/*
 * sioux.c - The main() for the sioux webserver; initializes and invokes
 *           the run loop in sioux_run.c.
 *
 */


#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <string.h>

#include <sthread.h>

#include <sioux_run.h>

/* Max length of a hostname */
const static size_t MAX_HOSTNAME = 128;

/* PORT_BASE should be > 1024 (ports below 1024 require root privalege) */
const static int PORT_BASE = 8000;
/* Restrict ports to PORT_BASE -> PORT_BASE+PORT_MAX_INCR */
const static int PORT_MAX_INCR = 8000;

/* The directory to look for files in */
const static char *DEFAULT_DOCROOT = "./docs";

static int web_getport(void);
static const char *web_gethostname(void);
static const char *web_getdocroot(void);
static void web_printurl(const char *host, int port);

int main(int argc, char **argv) {
  int port;
  const char *host;
  const char *docroot;

  sthread_init();

  /* Get the configuration information */
  host = web_gethostname();
  port = web_getport();
  docroot = web_getdocroot();

  /* Tell the user where to look for this server */
  web_printurl(host, port);

  /* Handle requests forever */
  web_runloop(host, port, docroot);
  return 0;
}

/* Try to guess a unique port (if there are lots of students
 * on the same machine, this helps avoid conflicts) */
int web_getport() {
  int port;
  port = PORT_BASE + (geteuid() % PORT_MAX_INCR);
  return port;
}

/* Get the hostname of this machine */
const char *web_gethostname() {
  char *name;
  name = (char*)malloc(sizeof(char)*MAX_HOSTNAME);
  assert(name != NULL);
  if (gethostname(name, MAX_HOSTNAME) == -1) {
    free(name);
    name = NULL;
    perror("sioux: unable to get hostname");
  }
  
  return name;
}

/* Return the path to the directory containing the files to serve */
const char *web_getdocroot() {
  /* Currently, no way to override default. */
  return DEFAULT_DOCROOT;
}

/* Print a URL the user can use to access this server */
void web_printurl(const char *host, int port) {
  host = (host==NULL) ? "UNKNOWN" : host;
  printf("starting sioux web server (%s threads) on:\n",
	 sthread_get_impl()==STHREAD_USER_IMPL ? "user" : "pthread" );
  printf("     http://%s:%d\n", host, port);
}
